@@ -69,7 +69,7 @@ static jstring _read_jstring(JNIEnv* env, BITBUFFER* buf, uint32_t length)
     char* str;
     _make_string(buf, &str, length);
     jstring jstr = (*env)->NewStringUTF(env, str);
-    free(str);
+    X_FREE(str);
     return jstr;
 }
 
@@ -220,10 +220,10 @@ static jobjectArray _parse_app_management_table(JNIEnv* env, BITBUFFER* buf)
 
         uint16_t name_data_length = bb_read(buf, 16);
         jobjectArray app_names = NULL;
+        int app_name_count = 0;
 
         if (name_data_length > 0) {
             // first scan for the number of app names
-            int app_name_count = 0;
             uint16_t name_bytes_read = 0;
             while (name_bytes_read < name_data_length) {
                 bb_seek(buf, 24, SEEK_CUR);
@@ -237,33 +237,34 @@ static jobjectArray _parse_app_management_table(JNIEnv* env, BITBUFFER* buf)
 
             // seek back to beginning of names
             bb_seek(buf, -name_data_length*8, SEEK_CUR);
-
-            app_names = bdj_make_array(env, "[Ljava/lang/String;", app_name_count);
-            JNICHK(app_names);
-
-            for (int j = 0; j < app_name_count; j++) {
-                char language[4];
-                _get_string(buf, language, 3);
-                jstring jlanguage = (*env)->NewStringUTF(env, language);
-                JNICHK(jlanguage);
-
-                uint8_t name_length = bb_read(buf, 8);
-                jstring jname = _read_jstring(env, buf, name_length);
-                JNICHK(jname);
-
-                jobjectArray app_name = bdj_make_array(env, "java/lang/String", 2);
-                JNICHK(app_name);
-
-                (*env)->SetObjectArrayElement(env, app_name, 0, jlanguage);
-                JNICHK(1);
-                (*env)->SetObjectArrayElement(env, app_name, 1, jname);
-                JNICHK(1);
-
-                (*env)->SetObjectArrayElement(env, app_names, j, app_name);
-                JNICHK(1);
-            }
         }
 
+        app_names = bdj_make_array(env, "[Ljava/lang/String;", app_name_count);
+        JNICHK(app_names);
+
+        for (int j = 0; j < app_name_count; j++) {
+            char language[4];
+            _get_string(buf, language, 3);
+            jstring jlanguage = (*env)->NewStringUTF(env, language);
+            JNICHK(jlanguage);
+
+            uint8_t name_length = bb_read(buf, 8);
+            jstring jname = _read_jstring(env, buf, name_length);
+            JNICHK(jname);
+
+            jobjectArray app_name = bdj_make_array(env, "java/lang/String", 2);
+            JNICHK(app_name);
+
+            (*env)->SetObjectArrayElement(env, app_name, 0, jlanguage);
+            JNICHK(1);
+            (*env)->SetObjectArrayElement(env, app_name, 1, jname);
+            JNICHK(1);
+
+            (*env)->SetObjectArrayElement(env, app_names, j, app_name);
+            JNICHK(1);
+        }
+
+
         // skip padding to word boundary
         if ((name_data_length & 0x1) != 0) {
             bb_seek(buf, 8, SEEK_CUR);
@@ -309,9 +310,9 @@ static jobjectArray _parse_app_management_table(JNIEnv* env, BITBUFFER* buf)
         uint8_t param_data_length = bb_read(buf, 8);
 
         jobjectArray params = NULL;
+        int param_count = 0;
         if (param_data_length > 0) {
             // first scan for the number of params
-            int param_count = 0;
             uint16_t param_bytes_read = 0;
             while (param_bytes_read < param_data_length) {
                 uint8_t param_length = bb_read(buf, 8);
@@ -323,19 +324,20 @@ static jobjectArray _parse_app_management_table(JNIEnv* env, BITBUFFER* buf)
 
             // seek back to beginning of params
             bb_seek(buf, -param_data_length*8, SEEK_CUR);
-
-            params = bdj_make_array(env, "java/lang/String", param_count);
-
-            for (int j = 0; j < param_count; j++) {
-                uint8_t param_length = bb_read(buf, 8);
-                jstring param = _read_jstring(env, buf, param_length);
-                JNICHK(param);
-
-                (*env)->SetObjectArrayElement(env, params, j, param);
-                JNICHK(1);
-            }
         }
 
+        params = bdj_make_array(env, "java/lang/String", param_count);
+
+        for (int j = 0; j < param_count; j++) {
+            uint8_t param_length = bb_read(buf, 8);
+            jstring param = _read_jstring(env, buf, param_length);
+            JNICHK(param);
+
+            (*env)->SetObjectArrayElement(env, params, j, param);
+            JNICHK(1);
+        }
+
+
         // skip padding to word boundary
         if ((param_data_length & 0x1) == 0) {
             bb_seek(buf, 8, SEEK_CUR);
@@ -429,10 +431,10 @@ jobject bdjo_read(JNIEnv* env, const char* file)
 
             result = _parse_bdjo(env, buf);
 
-            free(buf);
+            X_FREE(buf);
         }
 
-        free(data);
+        X_FREE(data);
     }
 
     file_close(handle);
