@@ -1,6 +1,7 @@
 /*
  * This file is part of libbluray
- * Copyright (C) 2010  William Hahne
+ * Copyright (C) 2010      William Hahne
+ * Copyright (C) 2012-2014 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,26 +25,33 @@ import java.awt.event.KeyEvent;
 import java.util.Vector;
 
 import javax.media.PackageManager;
+import javax.tv.service.SIManagerImpl;
 import javax.tv.service.selection.ServiceContextFactory;
 
+import org.bluray.bdplus.Status;
 import org.bluray.ti.DiscManager;
 import org.bluray.ti.TitleImpl;
 import org.bluray.ti.selection.TitleContext;
 import org.bluray.ui.event.HRcEvent;
 import org.dvb.event.EventManager;
+import org.dvb.io.ixc.IxcRegistry;
 import org.dvb.ui.FontFactory;
 import org.videolan.bdjo.Bdjo;
-import org.videolan.media.content.BDHandler;
+import org.videolan.media.content.PlayerManager;
 
 /**
  * This class allows BDJ to call various libbluray functions.
  */
 public class Libbluray {
-        protected static void init(long nativePointer, String discID) {
+    protected static void init(long nativePointer, String discID, String discRoot) {
+
+        System.setProperty("bluray.vfs.root", discRoot);
 
             Libbluray.nativePointer = nativePointer;
             DiscManager.getDiscManager().setCurrentDisc(discID);
 
+            BDJActionManager.createInstance();
+
             Vector prefix = new Vector();
             prefix.add("org.videolan");
             PackageManager.setContentPrefixList(prefix);
@@ -124,10 +132,14 @@ public class Libbluray {
     public static void shutdown() {
         try {
             BDJLoader.shutdown();
-            BDJActionManager.getInstance().finalize();
+            BDJActionManager.shutdown();
             MountManager.unmountAll();
             GUIManager.shutdown();
-            BDToolkit.shutdown();
+            BDToolkit.shutdownDisc();
+            SIManagerImpl.shutdown();
+            IxcRegistry.shutdown();
+            EventManager.shutdown();
+            Status.shutdown();
         } catch (Throwable e) {
             e.printStackTrace();
         }
@@ -199,11 +211,19 @@ public class Libbluray {
         return result;
     }
 
-    public static boolean selectPlaylist(int playlist) {
+    public static boolean selectPlaylist(int playlist, int playitem, int playmark, long time) {
         if (playlist < 0)
             throw new IllegalArgumentException("Playlist cannot be negative");
 
-        return selectPlaylistN(nativePointer, playlist) == 1 ? true : false;
+        return selectPlaylistN(nativePointer, playlist, playitem, playmark, time) == 1 ? true : false;
+    }
+
+    public static boolean selectPlaylist(int playlist) {
+        return selectPlaylist(playlist, -1, -1, -1);
+    }
+
+    public static void stopPlaylist() {
+        selectPlaylistN(nativePointer, -1, -1, -1, -1);
     }
 
     public static boolean selectTitle(TitleImpl title) {
@@ -260,7 +280,10 @@ public class Libbluray {
     }
 
     public static boolean selectRate(float rate) {
-        return selectRateN(nativePointer, rate) == 1 ? true : false;
+        return selectRateN(nativePointer, rate, 0) == 1 ? true : false;
+    }
+    public static boolean selectRate(float rate, boolean start) {
+        return selectRateN(nativePointer, rate, start ? 1 : 2) == 1 ? true : false;
     }
 
     public static void writeGPR(int num, int value) {
@@ -271,7 +294,11 @@ public class Libbluray {
     }
 
     public static void writePSR(int num, int value) {
-        int ret = writePSRN(nativePointer, num, value);
+        writePSR(num, value, 0xffffffff);
+    }
+
+    public static void writePSR(int num, int value, int psr_value_mask) {
+        int ret = writePSRN(nativePointer, num, value, psr_value_mask);
 
         if (ret == -1)
             throw new IllegalArgumentException("Invalid PSR");
@@ -311,32 +338,47 @@ public class Libbluray {
         int key = 0;
 
         switch (event) {
+
+        case BDJ_EVENT_START:
+            return BDJLoader.load(param);
+        case BDJ_EVENT_STOP:
+            return BDJLoader.unload();
+
         case BDJ_EVENT_CHAPTER:
-            BDHandler.onChapterReach(param);
+            PlayerManager.getInstance().onChapterReach(param);
             break;
         case BDJ_EVENT_MARK:
-            BDHandler.onMarkReach(param);
+            PlayerManager.getInstance().onMarkReach(param);
             break;
         case BDJ_EVENT_PLAYITEM:
-            BDHandler.onPlayItemReach(param);
+            PlayerManager.getInstance().onPlayItemReach(param);
+            break;
+        case BDJ_EVENT_PLAYLIST:
+            PlayerManager.getInstance().onPlaylistStart(param);
             break;
         case BDJ_EVENT_ANGLE:
-            BDHandler.onAngleChange(param);
+            PlayerManager.getInstance().onAngleChange(param);
             break;
         case BDJ_EVENT_SUBTITLE:
-            BDHandler.onSubtitleChange(param);
+            PlayerManager.getInstance().onSubtitleChange(param);
             break;
         case BDJ_EVENT_PIP:
-            BDHandler.onPiPChange(param);
+            PlayerManager.getInstance().onPiPChange(param);
             break;
+        case BDJ_EVENT_RATE:
+            float rate = (float)param / 90000.0f;
+            if (rate < 0.0f) rate = -rate;
+            if (rate < 0.01f) rate = 0.0f;
+            if (rate > 0.99f && rate < 1.01f) rate = 1.0f;
+            PlayerManager.getInstance().onRateChange(rate);
         case BDJ_EVENT_END_OF_PLAYLIST:
-            BDHandler.activePlayerEndOfMedia();
+            PlayerManager.getInstance().onPlaylistEnd(param);
             break;
         case BDJ_EVENT_PSR102:
             org.bluray.bdplus.Status.getInstance().receive(param);
             break;
         case BDJ_EVENT_PTS:
-            BDHandler.activePlayerUpdateTime(param);
+            PlayerManager.getInstance().onPlaylistTime(param);
             break;
         case BDJ_EVENT_VK_KEY:
             //case KeyEvent.KEY_TYPED:
@@ -359,6 +401,10 @@ public class Libbluray {
             case 14: key = KeyEvent.VK_LEFT; break;
             case 15: key = KeyEvent.VK_RIGHT; break;
             case 16: key = KeyEvent.VK_ENTER; break;
+            case 403: key = HRcEvent.VK_COLORED_KEY_0; break;
+            case 404: key = HRcEvent.VK_COLORED_KEY_1; break;
+            case 405: key = HRcEvent.VK_COLORED_KEY_2; break;
+            case 406: key = HRcEvent.VK_COLORED_KEY_3; break;
             default:
                 key = -1;
                 result = false;
@@ -392,6 +438,12 @@ public class Libbluray {
     private static final int BDJ_EVENT_VK_KEY                   = 8;
     private static final int BDJ_EVENT_MARK                     = 9;
     private static final int BDJ_EVENT_PSR102                   = 10;
+    private static final int BDJ_EVENT_PLAYLIST                 = 11;
+
+    private static final int BDJ_EVENT_START                    = 12;
+    private static final int BDJ_EVENT_STOP                     = 13;
+
+    private static final int BDJ_EVENT_RATE                     = 14;
 
     /* TODO: use org/bluray/system/RegisterAccess instead */
     public static final int PSR_IG_STREAM_ID     = 0;
@@ -444,7 +496,7 @@ public class Libbluray {
     private static native int getCurrentChapterN(long np);
     private static native long seekMarkN(long np, int mark);
     private static native long seekPlayItemN(long np, int clip);
-    private static native int selectPlaylistN(long np, int playlist);
+    private static native int selectPlaylistN(long np, int playlist, int playitem, int playmark, long time);
     private static native int selectTitleN(long np, int title);
     private static native int selectAngleN(long np, int angle);
     private static native void seamlessAngleChangeN(long np, int angle);
@@ -454,9 +506,9 @@ public class Libbluray {
     private static native int getCurrentAngleN(long np);
     private static native long tellN(long np);
     private static native long tellTimeN(long np);
-    private static native int selectRateN(long np, float rate);
+    private static native int selectRateN(long np, float rate, int reason);
     private static native int writeGPRN(long np, int num, int value);
-    private static native int writePSRN(long np, int num, int value);
+    private static native int writePSRN(long np, int num, int value, int psr_value_mask);
     private static native int readGPRN(long np, int num);
     private static native int readPSRN(long np, int num);
     private static native Bdjo getBdjoN(long np, String name);
